﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesignPattern.SingletonPattern.SingletonN
{
    public class SingletonN : IWorkItem
    {
        private static readonly int MaxInstances = 2;//最多能产生的实例个数
        private Status status = Status.Free;//初始状态
        public Status Status
        {
            get { return this.status; }
            set { this.status = value; }
        }
        public void DeActivate()
        {
            this.status = Status.Free;
        }
        //实现单件模式
        private static WorkItemCollection<SingletonN> collection = new WorkItemCollection<SingletonN>(MaxInstances);
        private static object lockHelper = new object();
        private SingletonN() { }
        //在Singleton主体框架不变下,引入集合实现多个实例的对象管理
        public static SingletonN Instance
        {
            get 
            {
                var instance = collection.GetWorkItem();
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            if (!collection.IsAddNewInstance) return null;
                            instance = new SingletonN();
                            collection.Add(instance);
                        }
                    }
                }
                instance.status = Status.Busy;//激活使用
                return instance;
            }
        }
    }

    #region 辅助对象
    /// <summary>
    /// 实例的执行状态
    /// </summary>
    public enum Status
    {
        Busy,//实例被客户程序占用
        Free//实例没有被客户程序占用
    }
    public interface IWorkItem
    {
        Status Status { get; set; }
        void DeActivate();
    }
    /// <summary>
    /// 定义Singleton实例集合
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class WorkItemCollection<T> where T : class, IWorkItem
    {
        protected int max;//定义最多保存的实例数量
        protected IList<T> items = new List<T>();
        public WorkItemCollection(int max)
        {
            this.max = max;
        }
        /// <summary>
        /// 外部获得T类型的入口
        /// </summary>
        /// <returns></returns>
        public virtual T GetWorkItem()
        {
            if (items == null || items.Count == 0) return null;
            //可能的话返回一个现成实例
            foreach (T item in items)
            {
                if (item.Status == Status.Free)
                {
                    item.Status = Status.Busy;
                    return item;
                }
            }
            return null;//如果没有现成的实例则返回null
        }
        /// <summary>
        /// 添加一个实例，默认状态为闲置
        /// </summary>
        /// <param name="item"></param>
        public virtual void Add(T item)
        {
            if (item == null) throw new ArgumentNullException("item is null");
            if (!IsAddNewInstance) throw new OverflowException();
            item.Status = Status.Free;
            items.Add(item);
        }
        /// <summary>
        /// 是否还能添加新的实例
        /// </summary>
        public virtual bool IsAddNewInstance
        {
            get { return items.Count < max; }
        }
    }
    #endregion
}
